------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.2"):new("TimeToDie")
local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0")

local _G = _G
local UIParent = UIParent
local UnitIsFriend = _G.UnitIsFriend
local UnitHealth = _G.UnitHealth
local GetTime = _G.GetTime
local CreateFrame = _G.CreateFrame

local defaultFont = _G.GameFontNormal:GetFont()
local p, frame, texture, text

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
	["TimeToDie"] = true,
	["Estimated time until current target will die."] = true,
	["Locked"] = true,
	["Toggle frame locked."] = true,
	["Enemy only"] = true,
	["Only track enemy targets."] = true,
	["Font"] = true,
	["Set font."] = true,
	["Font size"] = true,
	["Set font size."] = true,
	["Color"] = true,
	["Set font color."] = true,
	["Outline"] = true,
	["Set font outline."] = true,
	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
	["Strata"] = true,
	["Set frame strata."] = true,
	["High"] = true,
	["Medium"] = true,
	["Low"] = true,
	["Background"] = true,
	["Update period"] = true,
	["Set time between updates."] = true,
} end)

---------------------------------
--      Addon Declaration      --
---------------------------------

local TimeToDie = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
_G.TimeToDie = TimeToDie
TimeToDie.revision = tonumber(string.sub("$Revision", 12, -3))

---------------------------------
--      AceOptions Config      --
---------------------------------

local options = {
	name = L["TimeToDie"], type='group',
	desc = L["Estimated time until current target will die."],
	pass = true,
	get = "GetValue",
	set = "SetValue",
	args = {
		locked = {
			name = L["Locked"], type='toggle',
			desc = L["Toggle frame locked."],
			order = 100,
			disabled = "~IsActive",
		},
		enemy = {
			name = L["Enemy only"], type='toggle',
			desc = L["Only track enemy targets."],
			order = 110,
			disabled = "~IsActive",
		},
		font = {
			name = L["Font"], type='text',
			desc = L["Set font."],
			validate = SML and SML:List("font") or {defaultFont},
			set = nil,
			order = 110,
			disabled = "~IsActive",
		},
		size = {
			name = L["Font size"], type='range',
			desc = L["Set font size."],
			min = 4, max = 48, step = 1,
			order = 120,
			disabled = "~IsActive",
		},
		color = {
			name = L["Color"], type='color',
			desc = L["Set font color."],
			set = "SetColor",
			get = "GetColor",
			order = 130,
			disabled = "~IsActive",
		},
		outline = {
			name = L["Outline"], type='text',
			desc = L["Set font outline."],
			validate = {[""] = L["None"], OUTLINE = L["Normal"], THICKOUTLINE = L["Thick"]},
			order = 140,
			disabled = "~IsActive",
		},
		strata = {
			name = L["Strata"], type='text',
			desc = L["Set frame strata."],
			validate = {HIGH = L["High"], MEDIUM = L["Medium"], LOW = L["Low"], BACKGROUND = L["Background"]},
			order = 150,
			disabled = "~IsActive",
		},
		update = {
			name = L["Update period"], type='range',
			desc = L["Set time between updates."],
			min = .1, max = 5, step = .1,
			order = 160,
			disabled = "~IsActive",
		},
	},
}

------------------------------
--      Initialization      --
------------------------------

function TimeToDie:OnInitialize()
	TimeToDie:RegisterDB("TimeToDieDB")
	TimeToDie:RegisterDefaults("profile", {
		locked = false,
		enemy = true,
		font = defaultFont,
		size = 24,
		r = 1,
		g = .82,
		b = 0,
		outline = "",
		strata = "LOW",
		update = 1,
		
		p1 = "CENTER",
		p2 = "CENTER",
		x = 0,
		y = 0,
	})
	p = self.db.profile
	TimeToDie:RegisterChatCommand("/TimeToDie", options)
end

function TimeToDie:OnEnable(first)
	if first then
		self:CreateFrame()
	end
	
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_HEALTH")
	
	return frame:Show()
end

function TimeToDie:OnDisable()
	if frame then
		return frame:Hide()
	end
end

function TimeToDie:OnProfileEnable()
	p = self.db.profile
	if frame then
		return self:ApplySettings()
	end
end

-------------------------------
--      Event Functions      --
-------------------------------

function TimeToDie:UNIT_HEALTH(unit)
	if unit ~= "target" or p.enemy and UnitIsFriend("player", "target") or 
		self:IsEventScheduled("TimeToDieUpdate") then return end
	
	self:ScheduleRepeatingEvent("TimeToDieUpdate", function(self, time0, health0)
		local dhealth = health0 - UnitHealth("target")
		local dtime = GetTime() - time0
		local time = dhealth ~= 0 and UnitHealth("target") * dtime / dhealth or 0
		
		text:SetText(abs(time) > 60 and format("%dm : %ds", time/60, time%60) or format("%ds", time%60))
	end, p.update, self, GetTime(), UnitHealth("target"))
end

function TimeToDie:PLAYER_TARGET_CHANGED()
	if self:IsEventScheduled("TimeToDieUpdate") then
		self:CancelScheduledEvent("TimeToDieUpdate")
	end
	text:SetText(not p.locked and L["TimeToDie"] or nil)
end
------------------------------
--      Core Functions      --
------------------------------

function TimeToDie:CreateFrame()
	frame = _G.TimeToeDieFrame or CreateFrame("frame", "TimeToDieFrame", UIParent)
	frame:EnableMouse(true)
	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:SetWidth(120)
	frame:SetHeight(30)

	frame.isMoving = false
	frame:SetScript("OnDragStart", function(frame)
		frame:StartMoving()
		frame.isMoving = true
	end)
	frame:SetScript("OnDragStop", function(frame)
		if frame.isMoving then
			frame:StopMovingOrSizing()
			frame.isMoving = false
			p.p1, _, p.p2, p.x, p.y = frame:GetPoint()
		end
	end)

	texture = frame.texture or frame:CreateTexture("TimeToDieFrameTexture", "BACKGROUND")
	texture:SetTexture(0, 0, 0, 0.5)
	texture:ClearAllPoints()
	texture:SetAllPoints(frame)
	frame.texture = texture

	text = frame.text or frame:CreateFontString("TimeToDieFrameText", "OVERLAY", "GameFontNormal")
	text:SetFont(p.font, p.size, p.outline)
	text:SetJustifyH("CENTER")
	text:SetPoint("CENTER", frame)
	frame.text = text

	if (dewdrop) then
		dewdrop:Register(frame, 'children', function()
--~ 			dewdrop:AddLine('text', L["TimeToDie"], 'isTitle', true)
			dewdrop:FeedAceOptionsTable(options)
		end)
	end
	
	return self:ApplySettings()
end

function TimeToDie:GetValue(name)
	return p[name]
end

function TimeToDie:SetValue(name, v)
	p[name] = v
	return self:ApplySettings()
end

function TimeToDie:SetColor(r,g,b)
	p.r, p.g, p.b = r,g,b
	return self:ApplySettings()
end

function TimeToDie:GetColor()
	return p.r, p.g, p.b
end

function TimeToDie:ApplySettings()
	frame:SetFrameStrata(p.strata)
	frame:ClearAllPoints()
	frame:SetPoint(p.p1, UIParent, p.p2, p.x, p.y)
	if p.locked then
		texture:Hide()
		frame:RegisterForDrag()
		if text:GetText() == L["TimeToDie"] then
			text:SetText()
		end
	else
		texture:Show()
		frame:RegisterForDrag("LeftButton")
		if not text:GetText() then
			text:SetText(L["TimeToDie"])
		end
	end
	text:SetFont(SML and SML:Fetch("font", p.font) or defaultFont, p.size, p.outline)
	text:SetTextColor(p.r,p.g,p.b)
end
